//**************************************************************************
//**                                                                      **
//**            Copyright 1998 by Andrew D. Pitonyak                      **
//**                                                                      **
//**   This code belongs to me, and I make no claims of functionality.    **
//**   If you desire to use it, please discuss it with me, because I am   **
//**   curious how this code is used.                                     **
//**                                                                      **
//**   Oh yes, and any bugs that you fix, please let me know...           **
//**                                                                      **
//**                                Primatech, Inc.                       **
//**   Andrew D. Pitonyak           Andrew D. Pitonyak                    **
//**   4446 Mobile Drive #105       445 Hutchinson, Suite #200            **
//**   Columbus, OH 43220           Columbus, OH 43235                    **
//**   (614)-442-8615               (614)-841-9800                        **
//**                                andyp@primatech.com                   **
//**                                                                      **
//**************************************************************************
//**                                                                      **
//**  This module contains some structs which define some structures on   **
//**  in which we have some interest. This includes the partition table   **
//**  and enteries in the partition table.                                **
//**                                                                      **
//**************************************************************************

#if !defined(DRVSTRCT_HPP__)
#define DRVSTRCT_HPP__

#define DEFAULT_SECTOR_SIZE 512

//**************************************************************************
//**                                                                      **
//**  This item encapsulates a partition entry                            **
//**  An item that requires explanation is the SecCyl items. The sector   **
//**  is held in the bits 0--5 so simply "AND" the structure with 63 or   **
//**  0x3F. The cylinder is more difficult. Bits 15--8 contain the first  **
//**  8 bits and bits 7 and 6 are the two high bits.                      **
//**                                                                      **
//**************************************************************************

typedef struct {
   BYTE  status_;
   BYTE  startHead_;
   BYTE  startSecCyl_[2];
   BYTE  type_;
   BYTE  endHead_;
   BYTE  endSecCyl_[2];
   ULONG bootSec_;
   ULONG numSec_;
} PartitionEntryStruct;


//**************************************************************************
//**                                                                      **
//**  Structure that defines the boot/partition sector at                 **
//**  head 0, cylinder 0, sector 0 which is the first sector.             **
//**                                                                      **
//**  Address:                                                            **
//**  0x000    Boot code                                                  **
//**  0x1BE    16 BYTE Entry in the partition table                       **
//**  0x1CE    16 BYTE Entry in the partition table                       **
//**  0x1DE    16 BYTE Entry in the partition table                       **
//**  0x1DE    16 BYTE Entry in the partition table                       **
//**  0x1FE    2 BYTE IDcode(0xAA55) identifies partition sector          **
//**                                                                      **
//**************************************************************************

typedef struct {
   BYTE  bootCode_[0x1BE];
   PartitionEntryStruct entries_[4];
   WORD  iDCode_;
} PartitionSectorStruct;


//**************************************************************************
//**                                                                      **
//**  This is a structure which will account for the boot sector layout.  **
//**  Do not assume any null termination on any strings!                  **
//**                                                                      **
//**  Most of the fields are self explanatory. At offset 0x0E is a number **
//**  indicating the number of reserved sectors. This allows for extra    **
//**  sectors to follow the main boot sector which will contain extra     **
//**  boot code. Every version of PC DOS and OS/2 have only required 1    **
//**  but there may exist more than one on some systems, not that I am    **
//**  aware of any. A value of 1 means this is the last one.              **
//**  A big unknown is... Does this mean keep reading sectors until a 1   **
//**  is found in the proper location, or does it mean that i can simply  **
//**  look at this area and calculate where the fat is...                 **
//**                                                                      **
//**************************************************************************

typedef struct {
   BYTE  jmpBoot_[3];   // 0x00 jmp to E9xxx or EBxx90. 
   BYTE  name_[8];      // 0x03 Manufactures name and version number
   WORD  sectorSize_;   // 0x0B Bytes per sector
   BYTE  clusterSize_;  // 0x0D Sectors per Cluster
   WORD  reserveSect_;  // 0x0E Number of reserved sectors (see note: above)
   BYTE  fats_;         // 0x10 Number of fats
   WORD  rootDirSize_;  // 0x11 Number of entries in root directory
   WORD  volumeSize_;   // 0x13 Number of sectors in volume
   BYTE  descriptor_;   // 0x15 Media descriptor
   WORD  fatSize_;      // 0x16 Number of sectors per FAT
   WORD  trackSize_;    // 0x18 Sectors per track
   WORD  heads_;        // 0x1A Number of read/write heads
   WORD  hiddenSect_;   // 0x1C Number of hidden sectors
                        // This is purely guess work on my part!
                        // We are now in undocumented territory!
                        // Here starts some FAT32 stuff!
   WORD  hiddenSectHi_; // 0x1E Now hidden sectors is a ULONG
   ULONG bigTotalSect_; // 0x20 Total sectors on a FAT32 drive
   ULONG bigSectPerFat_;// 0x24 Number of sectors in a FAT
   WORD  extFlags_;     // 0x28 Bit 8 (0x0080) is for FAT mirroring.
                        //      If set then mirroring is disabled
                        //      so use the FAT indicated in the low
                        //      4 bits.
                        //      If bit 8 is clear then mirroring is
                        //      enabled and ALL FATS need to be updated.
                        //      Low 4 bits (0x000F) make up the 0 based
                        //      active FAT.
   WORD  fsVersion_;    // 0x2A File System version. [high byte].[low byte]
   ULONG rootStrtClus_; // 0x2C Starting cluster of Root Dir!
   WORD  fsInfoSect_;   // 0x30 File System info cluster which contains
                        //      the BIGFATBOOTFSINFO structure. If the
                        //      field is set to 0xFFFF then no structure
                        //      exists.
   WORD  bkUpBootSect_; // 0x32 Sector number of the backup boot sector.
                        //      If set to 0xFFFF then there is no sector.
   WORD  reserved_;     // 0x34 Actually MS says 6 of these!

   //??BYTE  unkown1_[6];   // 0x1E
   //??BYTE  physDrive_;    // 0x24 Physical Drive
   //??BYTE  unknown2_;     // 0x25
   //??BYTE  extBootSig_;   // 0x26 extended boot signature
   //??ULONG volSerialNum_; // 0x27 4 bytes long
   //??BYTE  volume_[11];   // 0x2B Volume title, probably space terminated
   BYTE  fileSystem_[8];// 0x36 File System String (FAT16, HPFS...)
   BYTE  unknown3_[0x1FD - 0x3E + 1]; // 0x3E
   WORD  signature_;    // 0x1FE This should be 0xAA55
   BYTE  bootRoutine_[0x1FF - 0x1E + 1]; // 0x1E Boot code
} BootSectorStruct;


//**************************************************************************
//**                                                                      **
//**  And just in case you want to use DosDevIOCtl category 8, func 0x63  **
//**                                                                      **
//**************************************************************************

typedef struct {
   WORD  sectorSize_;   // Bytes per sector
   BYTE  clusterSize_;  // Sectors per Cluster
   WORD  reserveSect_;  // Number of reserved sectors (see note: above)
   BYTE  fats_;         // Number of fats
   WORD  rootDirSize_;  // Number of entries in root directory
   WORD  volumeSize_;   // Number of sectors in volume
   BYTE  descriptor_;   // Media descriptor
   WORD  fatSize_;      // Number of sectors per FAT
   WORD  trackSize_;    // Sectors per track
   WORD  heads_;        // Number of read/write heads
   WORD  hiddenSect_;   // Number of hidden sectors
   ULONG sectors_;      // Total sectors;
   BYTE  reserved_[6];
} BPBStructOS2;



typedef struct {
   BPBStructOS2 bpb_;
   WORD  cylinders_;
   BYTE  type_;
   WORD  attrib_;
} QueryDeviceParamStruct;


//**************************************************************************
//**                                                                      **
//**  So what does a directory entry look on a FAT dir like?              **
//**  if the name starts with:                                            **
//**  0x00 - Last Dir Entry                                               **
//**  0x05 - First char is really 0xE5                                    **
//**  0x2E - File applies to current directory                            **
//**  0xE5 - The deleted                                                  **
//**                                                                      **
//**************************************************************************

#define FATTRIB_NOWRITE 0x01
#define FATTRIB_HIDDEN  0x02
#define FATTRIB_SYSTEM  0x04
#define FATTRIB_VOLUME  0x08
#define FATTRIB_SUBDIR  0x10
#define FATTRIB_ARCHIVE 0x20
#define FATTRIB_RESERVE 0xC0  // top two bits are reserved...

typedef struct {
   BYTE  name_[8];      // 0x00 first charcters define the entry type
   BYTE  ext_[3];       // 0x08
   BYTE  attrib_;       // 0x0B
   BYTE  reserved_[10]; // 0x0C
   WORD  time_;         // 0x16
   WORD  date_;         // 0x18
   WORD  cluster_;      // 0x1A
   ULONG size_;         // 0x1C
} FatDirEntryStruct;



#endif

